@@ -16,7 +16,15 @@ class Plugin(ad.Plugin):
 
     needs_plugins = ['lino_xl.lib.cal']
 
-    params_layout = """user assigned_to project event_type room state show_appointments"""
+    params_layout = """
+    navigation_panel
+    user
+    assigned_to
+    project
+    event_type
+    room
+    state
+    show_appointments"""
     """The actor parameter layout to use for filtering calendar views."""
 
     def setup_main_menu(self, site, user_type, m):
