@@ -75,6 +75,8 @@ class ParameterClone(dd.Actor):
     # clone_from = None
     # """subclasses must set this"""
 
+    params_panel_pos = 'left'
+
     @classmethod
     def class_init(cls):
         super(ParameterClone, cls).class_init()
@@ -278,7 +280,7 @@ class CalendarView(DayNavigator):
         return cls.detail_action
         # return dd.ShowDetail(cls.detail_layout)
 
-    @dd.htmlbox()
+    @dd.delayedhtmlbox()
     def navigation_panel(cls, obj, ar):
         if ar is None:
             return None
@@ -844,16 +846,16 @@ class MonthlySlave(ParameterClone, MonthlySlaveBase, DaysTable):
 
 class DayDetail(dd.DetailLayout):
     main = "body"
-    body = "navigation_panel:15 calview.DailySlave:85"
+    body = "calview.DailySlave:85"
 
 class WeekDetail(dd.DetailLayout):
     main = "body"
     # body = dd.Panel("navigation_panel:15 calview.WeeklySlave:85", label=_("Planner"))
-    body = "navigation_panel:15 calview.WeeklySlave:85"
+    body = "calview.WeeklySlave:85"
 
 class MonthDetail(dd.DetailLayout):
     main = "body"
-    body = "navigation_panel:15 calview.MonthlySlave:85"
+    body = "calview.MonthlySlave:85"
 
 class DailyView(ParameterClone, CalendarView):
     label = _("Daily view")
