@@ -1,38 +1,61 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2016-2020 Rumma & Ko Ltd
+# Copyright 2016-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-"""
-Model mixins for `lino_xl.lib.invoicing`.
-
-"""
-
+from django.db import models
+from datetime import timedelta
 from etgen.html import E, join_elems
 from lino.api import dd, _, gettext
 from lino.core.gfks import gfk2lookup
 from lino_xl.lib.ledger.utils import ZERO
 from lino_xl.lib.cal.utils import day_and_month
+from lino_xl.lib.cal.choicelists import DurationUnits
+from lino.utils import ONE_DAY
 
 from django.contrib.contenttypes.fields import GenericRelation
 
-MAX_SHOWN = 3  # maximum number of invoiced events shown in
-               # invoicing_info
+MAX_SHOWN = 3  # maximum number of invoiced events shown in invoicing_info
+
+class Periodicity(dd.Choice):
+    renew_every = None
+    renew_unit = None
+    renew_before = None
+
+    def __init__(self, value, text, renew_unit, renew_every, renew_before):
+        super(Periodicity, self).__init__(value, text, value)
+        self.renew_unit = renew_unit
+        self.renew_every = renew_every
+        self.renew_before = renew_before
+
+class Periodicities(dd.ChoiceList):
+    item_class = Periodicity
+
+add = Periodicities.add_item
+add("m", _("Monthly"), DurationUnits.months, 1, 7)
+add("q", _("Quarterly"), DurationUnits.months, 3, 14)
+add("y", _("Yearly"), DurationUnits.years, 1, 28)
+
 
 class InvoicingInfo(object):
     invoiced_qty = ZERO
     invoiced_events = 0
     used_events = []
     invoicings = None
+    invoicing_min_date = None
+    invoicing_max_date = None
     # tariff = None
+    # 20210724
     invoiceable_product = None
-    invoiceable_qty = None
+    # invoiceable_qty = None
     asset_to_buy = None
     number_of_events = None
     min_asset = None
     max_asset = None
 
-    def __init__(self, enr, max_date=None):
+    def __init__(self, enr, min_date, max_date):
+        # print("20210731 InvoicingInfo()", enr)
         self.generator = enr
+        self.min_date = min_date
         self.max_date = max_date
 
         max_date = self.max_date or dd.today()
@@ -40,21 +63,41 @@ class InvoicingInfo(object):
         if end_date:
             max_date = min(max_date, end_date)
 
-        product = enr.get_invoiceable_product(max_date)
-        # if not product:
-        #     # dd.logger.info("20181116c no product")
-        #     return
-
         start_date = enr.get_invoiceable_start_date(max_date)
 
-        # if hasattr(product, 'tariff'):
-        #     self.tariff = product.tariff
-        # self.tariff = getattr(product, 'tariff', None)
-        # tariff = product.tariff
-        # dd.logger.info("20181116 d %s %s", product, self.tariff)
-        # if self.tariff is None or (self.tariff.min_asset is None and
-        #                            self.tariff.max_asset) is None:
-        tariff = enr.get_invoiceable_tariff(product)
+        state_field = dd.plugins.invoicing.voucher_model._meta.get_field(
+            'state')
+        vstates = state_field.choicelist.get_editable_states()
+        qs = enr.invoicings.exclude(voucher__state__in=vstates)
+        product = enr.get_invoiceable_product(max_date)
+        if product is not None:
+            qs = qs.filter(product=product)
+        self.invoicings = qs
+
+        sub = enr.get_invoicing_periodicity(product)
+        if sub is not None:
+            last_invoicing = self.invoicings.last()
+            if last_invoicing is None:
+                # next_date = enr.get_invoiceable_start_date(max_date)
+                paid_until = start_date - ONE_DAY
+            else:
+                paid_until = last_invoicing.voucher.invoicing_max_date
+            next_date = paid_until - timedelta(days=sub.renew_before)
+            if next_date < max_date:
+                # next invoicing date not yet reached
+                return
+            self.asset_to_buy = 1
+            self.invoicing_min_date = paid_until + ONE_DAY
+            self.invoicing_max_date = sub.renew_unit.add_duration(
+                paid_until, sub.renew_every)
+            # self.invoiceable_product = product
+            return
+
+        self.invoiced_events = enr.get_invoiceable_free_events() or 0
+        self.used_events = list(enr.get_invoiceable_events(
+            start_date, max_date))
+
+        tariff = enr.get_invoicing_pass_type(product)
         if tariff is not None:
             self.number_of_events = tariff.number_of_events
             self.min_asset = tariff.min_asset
@@ -65,16 +108,6 @@ class InvoicingInfo(object):
             # dd.logger.info("20181116 e no tariff")
             # return
 
-        state_field = dd.plugins.invoicing.voucher_model._meta.get_field(
-            'state')
-        vstates = state_field.choicelist.get_editable_states()
-        qs = enr.invoicings.exclude(voucher__state__in=vstates)
-        if product is not None:
-            qs = qs.filter(product=product)
-        self.invoicings = qs
-
-        self.invoiced_events = enr.get_invoiceable_free_events() or 0
-
         for obj in self.invoicings:
             # tariff = getattr(obj.product, 'tariff', None)
             # if tariff:
@@ -82,35 +115,14 @@ class InvoicingInfo(object):
                 self.invoiced_qty += obj.qty
                 if self.number_of_events:
                     self.invoiced_events += int(obj.qty * self.number_of_events)
-            # history.append("".format())
         # print("20160414", self.invoicings, self.invoiced_qty)
 
-        # min_asset: the minimum number of events a customer should
-        # pay in advance
-
-        # max_asset : never invoice more than this number of events
-        # per month
-
-        # the asset is what the customer did already pay for.
-        # when the asset is negative, we want to invoice a quantity
-        # which sets the asset to min_asset.
-
         # print("20181116 f %s", self.tariff.number_of_events)
-        self.used_events = list(enr.get_invoiceable_events(
-            start_date, max_date))
-        asset = self.invoiced_events - len(self.used_events)
-
-        # dd.logger.info(
-        #     "20181119 %s %s %s %s",
-        #     start_date, max_date, asset, self.min_asset)
-
         if self.number_of_events:
-            # if not start_date:
-            #     return
-            # print("20160414 c", self.used_events)
-            # used_events = qs.count()
-            # paid_events = invoiced_qty * fee.number_of_events
-
+            asset = self.invoiced_events - len(self.used_events)
+            # dd.logger.info(
+            #     "20181119 %s %s %s %s",
+            #     start_date, max_date, asset, self.min_asset)
             if end_date and end_date < max_date and asset >= 0:
                 # ticket #1040 : a participant who declared to stop before
                 # their asset got negative should not get any invoice for
@@ -124,9 +136,6 @@ class InvoicingInfo(object):
             else:
                 self.asset_to_buy = self.min_asset - asset
 
-            # if asset > 0:
-            #     return
-
             if self.max_asset is not None:
                 self.asset_to_buy = min(self.asset_to_buy, self.max_asset)
 
@@ -137,6 +146,7 @@ class InvoicingInfo(object):
 
         # qty = self.asset_to_buy * enr.get_invoiceable_qty()
 
+        # 20210724
         self.invoiceable_product = product
         # self.invoiceable_qty = qty
         # self.asset_to_buy = asset_to_buy
@@ -189,6 +199,17 @@ class InvoicingInfo(object):
 
 
 
+# class InvoicingOrder(dd.Model):
+#
+#     class Meta:
+#         abstract = True
+#
+#     def get_invoicing_area(self):
+#         return rt.models.invoicing.Area.objects.first()
+#         # e.g. in Presto: return self.journal.room.invoicing_area
+#         # return None
+
+
 class InvoiceGenerator(dd.Model):
     # event_date_field = None
 
@@ -213,6 +234,13 @@ class InvoiceGenerator(dd.Model):
     #     # if isinstance(cls.invoiceable_date_field, six.string_types):
     #     #     cls.invoiceable_date_field =
 
+    def get_invoicing_area(self):
+        return rt.models.invoicing.Area.objects.first()
+        # raise NotImplementedError()
+        # return None
+        # no longer overridden in Presto: return self.journal.room.invoicing_area
+        # overridden in orders.Order
+
     def get_invoicings(self, **kwargs):
         # deprecated. use invoicings instead.
         item_model = dd.plugins.invoicing.item_model
@@ -227,8 +255,9 @@ class InvoiceGenerator(dd.Model):
         return True
 
     def get_invoice_items(self, info, invoice, ar):
-        # every generator produces one invoice item per invoicing plan
-        # print("20190328", info.invoiceable_product, info.number_of_events)
+
+        # print("20210727 get_invoice_items()", self, info.invoiceable_product, info.number_of_events)
+        # 20210724
         if info.invoiceable_product is None:
             return
 
@@ -236,52 +265,66 @@ class InvoiceGenerator(dd.Model):
         if invoiceable_qty is None:
             return
 
-        kwargs = dict(invoiceable=self, product=info.invoiceable_product)
+        # 20210724
+        kwargs = dict(product=info.invoiceable_product)
+        # 20210804 kwargs = dict(invoiceable=self, product=info.invoiceable_product)
+        # kwargs = dict(invoiceable=self)
+
+        asset_to_buy = info.asset_to_buy
+        if asset_to_buy is None:
+            return
 
         if info.number_of_events is None:
-            qty = info.asset_to_buy * invoiceable_qty
+            qty = asset_to_buy * invoiceable_qty
             kwargs.update(
                 title=self.get_invoiceable_title(), qty=qty)
-            yield invoice.add_voucher_item(**kwargs)
+            i = invoice.add_voucher_item(**kwargs)
+            i.discount_changed(ar)
+            yield i
             return
 
         # sell the asset in chunks
-        asset_to_buy = info.asset_to_buy
+
         number = info.invoiced_events // info.number_of_events
-        # number = 0
         while asset_to_buy > 0:
             number += 1
             kwargs.update(
                 title=self.get_invoiceable_title(number), qty=invoiceable_qty)
-            yield invoice.add_voucher_item(**kwargs)
+            i = invoice.add_voucher_item(**kwargs)
+            i.discount_changed(ar)
+            yield i
             asset_to_buy -= info.number_of_events
 
     def get_invoiceable_title(self, number=None):
         return str(self)
 
-    def compute_invoicing_info(self, max_date=None):
+    def compute_invoicing_info(self, min_date, max_date):
         if self._invoicing_info is None \
+           or self._invoicing_info.min_date != min_date \
            or self._invoicing_info.max_date != max_date:
-            self._invoicing_info = InvoicingInfo(self, max_date)
-        # assert self._invoicing_info.max_date == max_date
+            self._invoicing_info = InvoicingInfo(self, min_date, max_date)
+        # assert self._invoicing_info.generator is self
         return self._invoicing_info
 
     @dd.displayfield(_("Invoicing info"))
     def invoicing_info(self, ar):
-        info = self.compute_invoicing_info(dd.today())
+        info = self.compute_invoicing_info(None, dd.today())
         return info.format_as_html(ar)
 
-    # def get_invoiceable_info(self, max_date=None):
-    #     return self.compute_invoicing_info(max_date)
-
     def get_invoiceable_product(self, max_date=None):
         return None
 
-    def get_invoiceable_tariff(self, product=None):
+    def get_invoiceable_qty(self):
+        return self.default_invoiceable_qty
+
+    def get_invoicing_pass_type(self, product=None):
         if product is not None:
             return product.tariff
         return None
 
+    def get_invoicing_periodicity(self, product):
+        return
+
     def get_invoiceable_end_date(self):
         return self.end_date
 
@@ -303,9 +346,6 @@ class InvoiceGenerator(dd.Model):
     def get_invoiceable_free_events(self):
         return 0
 
-    def get_invoiceable_qty(self):
-        return self.default_invoiceable_qty
-
     # def get_invoiceable_amount(self, ie):
     #     return ie.amount
 
@@ -326,7 +366,24 @@ class InvoiceGenerator(dd.Model):
 
     @classmethod
     def get_generators_for_plan(cls, plan, partner=None):
-        return []
+        return cls.objects.all()
+
+    def setup_invoice_from_suggestion(self, invoice, plan, info):
+        if info.invoicing_max_date is not None:
+            invoice.invoicing_min_date = info.invoicing_min_date
+            invoice.invoicing_max_date = info.invoicing_max_date
+        else:
+            invoice.invoicing_min_date = plan.min_date
+            invoice.invoicing_max_date = plan.get_max_date()
 
     def setup_invoice_item(self, item):
         pass
+
+    @classmethod
+    def filter_by_invoice_recipient(cls, qs, partner, fieldname):
+        q1 = models.Q(**{
+            fieldname + '__salesrule__invoice_recipient__isnull': True,
+            fieldname:partner})
+        q2 = models.Q(**{
+            fieldname+'__salesrule__invoice_recipient':partner})
+        return qs.filter(models.Q(q1 | q2))
