@@ -13,7 +13,7 @@ from lino.utils.report import Report
 from .choicelists import TradeTypes, VoucherTypes, JournalGroups
 from .choicelists import VoucherStates
 from .mixins import JournalRef, PeriodRangeObservable
-from .roles import AccountingReader, LedgerUser, LedgerStaff
+from .roles import AccountingReader, LedgerUser, LedgerStaff, LedgerPartner
 from .utils import Balance
 from .utils import DC, ZERO
 
@@ -66,6 +66,7 @@ class JournalsOverview(Journals):
     display_mode = "summary"
     editable = False
 
+
     @classmethod
     def get_table_summary(cls, mi, ar):
         if ar is None:
@@ -1087,6 +1088,27 @@ class MovementsByPartner(Movements):
         return ar.html_text(E.div(*elems))
 
 
+class MyMovements(Movements):
+    editable = True
+    allow_create = False
+    label = _("My movements")
+    required_roles = dd.login_required(LedgerPartner)
+    column_names = 'value_date voucher_link debit credit match cleared *'
+    order_by = dd.plugins.ledger.remove_dummy(
+        '-value_date', 'voucher__id', 'account__ref', 'project', 'id')
+    params_layout = """
+    start_period end_period start_date end_date cleared"""
+
+    @classmethod
+    def get_filter_kw(self, ar, **kw):
+        p = ar.get_user().partner
+        if p is None:
+            return None
+        kw.update(partner=p)
+        return super(MyMovements, self).get_filter_kw(ar, **kw)
+
+
+
 class MovementsByProject(MovementsByPartner):
     editable = False
     column_names = 'value_date voucher_link description \
@@ -1095,6 +1117,7 @@ class MovementsByProject(MovementsByPartner):
     display_mode = "html"
     order_by = ['-value_date', 'partner', 'id']
 
+
     @classmethod
     def param_defaults(cls, ar, **kw):
         kw = super(MovementsByPartner, cls).param_defaults(ar, **kw)
