@@ -187,7 +187,7 @@ def setup_memo_commands(sender=None, **kwargs):
 
     Meeting = sender.models.meetings.Meeting
 
-    def cmd(parser, s):
+    def cmd(parser, s, cmdname, usages):
         pk = s
         txt = None
 
@@ -211,4 +211,3 @@ def setup_memo_commands(sender=None, **kwargs):
         cmd=cmd,
         # title=lambda obj: obj.name
     )
-
