@@ -27,4 +27,4 @@ from lino.modlib.users import Plugin
 class Plugin(Plugin):
     needs_plugins = ['lino_xl.lib.countries']
     extends_models = ['User']
-    online_registration = True
+    # online_registration = True
