@@ -65,9 +65,26 @@ class Page(Hierarchical, Sequenced, Previewable, Publishable):
         _("Reference"), max_length=200, blank=True, null=True)
     language = LanguageField(default=models.NOT_PROVIDED, blank=True)
     title = dd.CharField(_("Title"), max_length=250, blank=True)
+    child_node_depth = models.IntegerField(default=0)
     # body = dd.BabelTextField(_("Body"), blank=True, format='plain')
     # raw_html = models.BooleanField(_("raw html"), default=False)
 
+    def get_child_nodes_in_order(self):
+
+        def get_next_layer(depth, parent, qs):
+            if depth >= 1:
+                objs = self.__class__.objects.filter(parent=parent).order_by('seqno')
+                if objs.count():
+                    for obj in objs:
+                        qs.append(obj)
+                        depth_from_parent = depth - 1
+                        if obj.child_node_depth > depth_from_parent:
+                            depth_from_parent = obj.child_node_depth
+                        qs.extend(get_next_layer(depth_from_parent, obj, []))
+            return qs
+
+        return get_next_layer(self.child_node_depth, self, [])
+
     @classmethod
     def lookup_page(cls, ref):
         try:
@@ -158,7 +175,7 @@ class Page(Hierarchical, Sequenced, Previewable, Publishable):
 class PageDetail(dd.DetailLayout):
     main = """
     ref language parent seqno
-    title
+    title child_node_depth
     body:70 pages.PagesByParent:30
     """
 
