@@ -27,7 +27,7 @@ class SalesDocument(VatVoucher, Certifiable):
 
     language = dd.LanguageField()
     subject = models.CharField(_("Subject line"), max_length=200, blank=True)
-    intro = models.TextField("Introductive Text", blank=True)
+    intro = models.TextField(_("Introductive text"), blank=True)
     paper_type = dd.ForeignKey('sales.PaperType', null=True, blank=True)
     # channel = Channels.field(default=Channels.as_callable('paper'))
 
@@ -42,8 +42,8 @@ class SalesDocument(VatVoucher, Certifiable):
         return TradeTypes.sales
 
     def add_voucher_item(self, product=None, qty=None, **kw):
-        Product = rt.models.products.Product
         if product is not None:
+            Product = rt.models.products.Product
             if not isinstance(product, Product):
                 product = Product.objects.get(pk=product)
             # if qty is None:
