@@ -37,6 +37,11 @@ site_model = dd.plugins.tickets.site_model
 milestone_model = dd.plugins.tickets.milestone_model
 end_user_model = dd.plugins.tickets.end_user_model
 
+if dd.is_installed('invoicing'):
+    order_model = dd.plugins.invoicing.order_model
+else:
+    order_model = None
+
 # if dd.is_installed('tickets'):
 #     site_model = dd.plugins.tickets.site_model
 #     milestone_model = dd.plugins.tickets.milestone_model
@@ -181,6 +186,7 @@ class TicketType(mixins.BabelNamed):
 
 
 class Site(Referrable, ContactRelated, Starrable, DateRange):
+    # ContactRelated for backwards compatibility
     class Meta:
         app_label = 'tickets'
         # verbose_name = pgettext("Ticketing", "Site")
@@ -213,6 +219,8 @@ class Site(Referrable, ContactRelated, Starrable, DateRange):
     group = dd.ForeignKey('groups.Group', null=True, blank=True)
     hours_paid = dd.DurationField(_("Hours paid"), blank=True, null=True)
     private = models.BooleanField(_("Private"), default=False)
+    order = dd.ForeignKey(order_model, null=True, blank=True)
+        # order = dd.ForeignKey(dd.plugins.invoicing.order_model, null=True, blank=True)
 
     def __str__(self):
         return self.ref or self.name
@@ -881,7 +889,7 @@ def setup_memo_commands(sender=None, **kwargs):
     mp.add_suggester(
         "#", sender.models.tickets.Ticket.objects.order_by('id'), 'id')
 
-    def py2html(parser, s):
+    def py2html(parser, s, cmdname, usages):
         url, txt = py2url_txt(s)
         if url:
             # lines = inspect.getsourcelines(s)
