@@ -3,47 +3,18 @@
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 
-import random
 from lino.utils import Cycler
 from lino.api import rt
 from lino_xl.lib.vat.choicelists import VatAreas, VatRegimes, VatRules
 
-# Thanks to https://www.avalara.com/vatlive/en/eu-vat-rules/eu-vat-number-registration/eu-vat-number-formats.html
-vat_id_lengths = {
-    "AT": 8,
-    "BE": 10,
-    "HR": 11,
-    "DK": 8,
-    "EE": 10,
-    "FI": 8,
-    "FR": 11,
-    "DE": 9,
-    "NL": 9,
-    "EL": 9,
-    "HU": 8,
-    "IT": 11,
-    "LV": 11,
-    "LT": 12,
-    "LU": 8,
- }
-
 
 def objects():
-    random.seed(1)  # we want always the same sequence of random numbers
     qs = rt.models.contacts.Company.objects.filter(country__isnull=False)
     qs = qs.filter(vat_id="")
     for obj in qs:
-        isocode = obj.country.isocode
-        length = vat_id_lengths.get(isocode, None)
-        if not length:
-            # raise Exception("20190408 no VAT id length for {}".format(isocode))
+        if not obj.vid_manager.origin.length:
             continue
-        number = str(random.randint(int("1" * length), int("9" * length)))
-        if isocode == "AT":
-            number = "U" + number
-        elif isocode == "NL":
-            number = number + "B01"
-        obj.vat_id = "{}{}".format(isocode, number)
+        obj.vid_manager.generate_vid()
         yield obj
 
     va2regimes = dict()
